/****************************************************************************
                             Hyper's CD Catalog
		A multiplatform qt and xml based catalog program
 
 Author    : Christoph Thielecke (crissi99@gmx.de)
 License   : GPL
 Copyright : (C) 2003 Christoph Thielecke
****************************************************************************/


#include <qvariant.h>
#include <qpushbutton.h>
#include <qlabel.h>
#include <qlistbox.h>
#include <qcheckbox.h>
#include <qbuttongroup.h>
#include <qradiobutton.h>
#include <qlineedit.h>
#include <qlayout.h>
#include <qpixmap.h>
#include <qtooltip.h>
#include <qmessagebox.h>
#include <qtextstream.h>
#include <qapplication.h>
#include <qstring.h>

#include "icons.h"
#include "cdcat.h"
#include "exportcdcatdb.h"

/*
 *  Constructs a exportCdcatDB as a child of 'parent', with the
 *  name 'name' and widget flags set to 'f'.
 *
 *  The dialog will by default be modeless, unless you set 'modal' to
 *  TRUE to construct a modal dialog.
 */
exportCdcatDB::exportCdcatDB(DataBase *db, QWidget* parent, const char* name, bool modal, WFlags fl) : QDialog( parent, name, modal, fl )
{
	p = db;

	if ( !name )
		setName( "exportCdcatDB" );
	setSizeGripEnabled( TRUE );
	exportCdcatDBLayout = new QGridLayout( this, 1, 1, 11, 6, "exportCdcatDBLayout");

	layout32 = new QVBoxLayout( 0, 0, 6, "layout32");

	layout21 = new QHBoxLayout( 0, 0, 6, "layout21");

	layout10_2 = new QVBoxLayout( 0, 0, 6, "layout10_2");

	textLabel2 = new QLabel( this, "textLabel2" );
	layout10_2->addWidget( textLabel2 );

	allMedia = new QListBox( this, "allMedia" );
	allMedia->setSelectionMode( QListBox::Extended );
	layout10_2->addWidget( allMedia );
	layout21->addLayout( layout10_2 );

	layout19 = new QVBoxLayout( 0, 0, 6, "layout19");
	QSpacerItem* spacer = new QSpacerItem( 20, 180, QSizePolicy::Minimum, QSizePolicy::Expanding );
	layout19->addItem( spacer );

	layout10 = new QHBoxLayout( 0, 0, 6, "layout10");

	buttonRemove = new QPushButton( this, "buttonRemove" );
	buttonRemove->setMinimumSize( QSize( 50, 23 ) );
	buttonRemove->setMaximumSize( QSize( 50, 23 ) );
	buttonRemove->setPixmap( *get_t_remove_export_icon() );
	layout10->addWidget( buttonRemove );

	buttonAdd = new QPushButton( this, "buttonAdd" );
	buttonAdd->setMinimumSize( QSize( 50, 23 ) );
	buttonAdd->setMaximumSize( QSize( 50, 23 ) );
	buttonAdd->setPixmap( *get_t_add_export_icon() );
	layout10->addWidget( buttonAdd );
	layout19->addLayout( layout10 );
	QSpacerItem* spacer_2 = new QSpacerItem( 20, 210, QSizePolicy::Minimum, QSizePolicy::Expanding );
	layout19->addItem( spacer_2 );
	layout21->addLayout( layout19 );

	layout10_2_2 = new QVBoxLayout( 0, 0, 6, "layout10_2_2");

	textLabel2_2 = new QLabel( this, "textLabel2_2" );
	layout10_2_2->addWidget( textLabel2_2 );

	selectedMedia = new QListBox( this, "selectedMedia" );
	selectedMedia->setSelectionMode( QListBox::Extended );
	layout10_2_2->addWidget( selectedMedia );
	layout21->addLayout( layout10_2_2 );
	layout32->addLayout( layout21 );

	checkAllMedia = new QCheckBox( this, "checkAllMedia" );
	layout32->addWidget( checkAllMedia );

	buttonGroup1 = new QButtonGroup( this, "buttonGroup1" );
	buttonGroup1->setColumnLayout(0, Qt::Vertical );
	buttonGroup1->layout()->setSpacing( 6 );
	buttonGroup1->layout()->setMargin( 11 );
	buttonGroup1Layout = new QGridLayout( buttonGroup1->layout() );
	buttonGroup1Layout->setAlignment( Qt::AlignTop );

	layout26 = new QVBoxLayout( 0, 0, 6, "layout26");

	radioHtml = new QRadioButton( buttonGroup1, "radioHtml" );
	layout26->addWidget( radioHtml );

	layout25 = new QHBoxLayout( 0, 0, 6, "layout25");

	radioCsv = new QRadioButton( buttonGroup1, "radioCsv" );
	layout25->addWidget( radioCsv );
	QSpacerItem* spacer_3 = new QSpacerItem( 130, 20, QSizePolicy::Expanding, QSizePolicy::Minimum );
	layout25->addItem( spacer_3 );

	textLabel1 = new QLabel( buttonGroup1, "textLabel1" );
	layout25->addWidget( textLabel1 );

	separatorInput = new QLineEdit( buttonGroup1, "separatorInput" );

#ifndef _WIN32

	separatorInput->setSizePolicy( QSizePolicy( (QSizePolicy::SizeType)0, (QSizePolicy::SizeType)0, 0, 0, separatorInput->sizePolicy().hasHeightForWidth() ) );
#endif

	separatorInput->setMaximumSize( QSize( 25, 32767 ) );
	layout25->addWidget( separatorInput );
	layout26->addLayout( layout25 );

	buttonGroup1Layout->addLayout( layout26, 0, 0 );
	layout32->addWidget( buttonGroup1 );

	layout29 = new QHBoxLayout( 0, 0, 6, "layout29");

	textLabel3 = new QLabel( this, "textLabel3" );
	layout29->addWidget( textLabel3 );

	fileName = new QLineEdit( this, "fileName" );
	layout29->addWidget( fileName );

	buttonFile = new QPushButton( this, "buttonFile" );
	layout29->addWidget( buttonFile );
	layout32->addLayout( layout29 );

	layout4 = new QHBoxLayout( 0, 0, 6, "layout4");
	QSpacerItem* spacer_4 = new QSpacerItem( 163, 20, QSizePolicy::Expanding, QSizePolicy::Minimum );
	layout4->addItem( spacer_4 );

	buttonOk = new QPushButton( this, "buttonOk" );
	buttonOk->setAutoDefault( TRUE );
	buttonOk->setDefault( TRUE );
	layout4->addWidget( buttonOk );

	buttonCancel = new QPushButton( this, "buttonCancel" );
	buttonCancel->setAutoDefault( TRUE );
	layout4->addWidget( buttonCancel );
	QSpacerItem* spacer_5 = new QSpacerItem( 163, 20, QSizePolicy::Expanding, QSizePolicy::Minimum );
	layout4->addItem( spacer_5 );
	layout32->addLayout( layout4 );

	exportCdcatDBLayout->addLayout( layout32, 0, 0 );
	languageChange();
	resize( QSize(418, 389).expandedTo(minimumSizeHint()) );
	clearWState( WState_Polished );

	// signals and slots connections
	connect( buttonOk, SIGNAL( clicked() ), this, SLOT( ok() ) );
	connect( buttonCancel, SIGNAL( clicked() ), this, SLOT( cancel() ) );
	connect( buttonAdd, SIGNAL( clicked() ), this, SLOT( addMedia() ) );
	connect( buttonRemove, SIGNAL( clicked() ), this, SLOT( removeMedia() ) );
	connect( checkAllMedia, SIGNAL( toggled(bool) ), this, SLOT( allMediaToggled() ) );
	connect( buttonFile, SIGNAL(clicked() ),this, SLOT (browseFile()));
	connect( radioCsv, SIGNAL (clicked()),this, SLOT (doCsvExport()));
	connect( radioHtml, SIGNAL (clicked()),this, SLOT (doHtmlExport()));

	separatorInput->setText(";");
	radioCsv->setChecked(true);

	outstring="";
	medianame="";
	currentpath="";
	out1=NULL;
	out2=NULL;
	out3=NULL;
	out4=NULL;
	out5=NULL;

	//checkAllMedia->setChecked(true);
	//checkAllMedia->setEnabled(false);
	fillMedia();

	//tmp
	//fileName->setText("/data/test.txt");
}

/*
 *  Destroys the object and frees any allocated resources
 */
exportCdcatDB::~exportCdcatDB()
{
	// no need to delete child widgets, Qt does it all for us
}

/*
 *  Sets the strings of the subwidgets using the current
 *  language.
 */
void exportCdcatDB::languageChange()
{
	setCaption( tr( "exportCdcatDB database" ) );
	textLabel2->setText( tr( "Availiable media" ) );
	QToolTip::add
		( allMedia, tr( "Add media" ) );
	buttonRemove->setText( QString::null );
	QToolTip::add
		( buttonRemove, tr( "Remove media" ) );
	buttonAdd->setText( QString::null );
	QToolTip::add
		( buttonAdd, tr( "Add media" ) );
	textLabel2_2->setText( tr( "Media to export" ) );
	QToolTip::add
		( selectedMedia, tr( "Media to export" ) );
	checkAllMedia->setText( tr( "All media" ) );
	buttonGroup1->setTitle( tr( "Type" ) );
	radioHtml->setText( tr( "export as HTML" ) );
	radioCsv->setText( tr( "export as CSV" ) );
	textLabel1->setText( tr( "separator:" ) );
	textLabel3->setText( tr( "File to export:" ) );
	buttonFile->setText( tr( "..." ) );
	buttonOk->setText( tr( "&OK" ) );
#ifndef _WIN32

	buttonOk->setAccel( QKeySequence( QString::null ) );
#endif

	buttonCancel->setText( tr( "&Cancel" ) );

#ifndef _WIN32

	buttonCancel->setAccel( QKeySequence( QString::null ) );
#endif
}

void exportCdcatDB::fillMedia()
{
        if(p == NULL)
	 {
	   buttonOk     ->setEnabled(false);;
	   allMedia     ->setEnabled(false);
	   selectedMedia->setEnabled(false);
	   buttonAdd->setEnabled(false);
	   buttonRemove ->setEnabled(false);
	   buttonFile   ->setEnabled(false);
	   fileName     ->setEnabled(false);
	   checkAllMedia->setEnabled(false);
	   return;
	 }
	Node *source = p->getRootNode();
	setCaption(tr("exportCdcatDB database: ")+((DBCatalog *)(source->data))->name);
	Node *tmp;
	tmp=source->child;

	while(tmp != NULL)
	{
		if(tmp->type != HC_MEDIA )
		{
			tmp=tmp->next;
			continue;
		}

		this->allMedia->insertItem(QString(tmp->getNameOf()));
		tmp=tmp->next;
	}

    allMedia->sort();
}

bool exportCdcatDB::doHtmlExport()
{
	//fileName->setText("/data/test.html");
	return true;
}

bool exportCdcatDB::doCsvExport()
{
	//fileName->setText("/data/test.txt");
	return true;
}

void exportCdcatDB::ok()
{
	if(radioHtml->isChecked())
	{
		outstring+="<html>\n";
		outstring+=" <!-- This file was generated by CdCat";
		outstring+=HOMEPAGE;
		outstring+=" -->\n";
		outstring+="<head>\n<title>"+tr("Export from cdcat:")+" "+((DBCatalog *)((p->getRootNode())->data))->name+"</title>\n</head>\n";
		outstring+="<body>\n<h1>"+tr("Export from cdcat:")+" "+((DBCatalog *)((p->getRootNode())->data))->name+"</h1>\n";
		outstring+="<table border=\"1\"><tr><td><strong>"+tr("Media")+"</strong></td><td><strong>"+tr("File");
		outstring+="</strong></td><td><strong>"+tr("Size")+"</strong></td><td><strong>"+tr("Date")+"</strong></td></tr>";
	}
	if(radioCsv->isChecked())
	{}

	if ((fileName->text()).isEmpty())
		QMessageBox::critical(0,tr("Filename missing"),tr("Please enter a filename!"));
	else if((separatorInput->text()).isEmpty())
		QMessageBox::critical(0,tr("Separator missing"),tr("Please enter a separator!"));
	else
	{
		QApplication::setOverrideCursor(waitCursor);

		separator=separatorInput->text();

                /*extension correction:*/
                if(radioHtml->isChecked() &&
		    !( ((fileName->text()).right(5)).lower() == ".html"  ||
		       ((fileName->text()).right(4)).lower() == ".htm"      ) )
              	 {
	           QString q(fileName->text());
		   fileName->setText(q.append(".html"));       
      	         }
		 
		if(radioCsv->isChecked() &&
		     ((fileName->text()).right(4)).lower() != ".csv"  )
              	 {
	           QString q(fileName->text());
		   fileName->setText(q.append(".csv"));       
      	         } 

		f.setName(fileName->text());
		if (f.open(IO_WriteOnly))
		{
			QTextStream str(&f);    // we will serialize the data into file f
			str.setEncoding(QTextStream::Latin1);

			//str << " # output generated by cdcat" << endl;
			writeDown(p->getRootNode());

			if(radioHtml->isChecked())
				outstring+="</table></body></html>";

			if(radioCsv->isChecked())
			{}


			str << outstring;
			f.close();

			QApplication::restoreOverrideCursor();

		}
		else
			QMessageBox::critical(0,tr("Error during write config..."),tr("I can't create or rewrite the file"),tr("Ok"),0);
		close();
	}
 

}

void exportCdcatDB::cancel()
{
	close();
}

void exportCdcatDB::addMedia()
{
    uint i;
    for (i=0; i < allMedia->count();i++)
 	if (allMedia->isSelected(i))
 	     selectedMedia->insertItem(allMedia->text(i));


    for (i=0; i < allMedia->count();i++)
	while(i < allMedia->count() && allMedia->isSelected(i))
	     allMedia->removeItem(i);
    allMedia->sort();
    selectedMedia->sort();
    	     
}

void exportCdcatDB::removeMedia()
{
    uint i;
    for (i=0; i < selectedMedia->count();i++)
	if (selectedMedia->isSelected(i))
     	     allMedia->insertItem(selectedMedia->text(i));


    for (i=0; i < selectedMedia->count();i++)
	while(i < selectedMedia->count() &&  selectedMedia->isSelected(i))
	     selectedMedia->removeItem(i);
    allMedia->sort();
    selectedMedia->sort();
	     
}

void exportCdcatDB::allMediaToggled()
{
	if (checkAllMedia->isChecked())
	{
		allMedia->setEnabled(false);
		selectedMedia->setEnabled(false);
		buttonAdd->setEnabled(false);
		buttonRemove->setEnabled(false);
	}
	else
	{
		allMedia->setEnabled(true);
		selectedMedia->setEnabled(true);
		buttonAdd->setEnabled(true);
		buttonRemove->setEnabled(true);
	}
}

void exportCdcatDB::browseFile()
{
	QString s;
	s = QFileDialog::getSaveFileName(QString::null,tr("All files (*.*)"),this,
	                                 tr("file chooser"),tr("Choose a file!"));
	if(!s.isEmpty())
		fileName->setText(s);
}

int  exportCdcatDB::writeDown(Node *source)
{
	int i=0;
	switch(source->type)
	{
	case 0:
		return 1;
	case 1:
		i=writeCatalog(source);
		break;
	case 2:
		i=writeMedia(source);
		break;
	case 3:
		i=writeDirectory(source);
		break;
	case 4:
		i=writeFile(source);
		break;
	case 5:
		i=writeMp3Tag(source);
		break;
	}
	return i;
}

int  exportCdcatDB::writeCatalog(Node *source)
{
	/*
	outstring+="Name: "+QString(recode(((DBCatalog *)(source->data))->name,&out1))+", ";
	outstring+="Owner: "+QString(recode(((DBCatalog *)(source->data))->owner,&out2))+", ";
	outstring+="Modified: "+QString(recode(((DBCatalog *)(source->data))->modification,&out3))+", ";
	*/

	//  commentWriter(((DBCatalog *)(source->data))->comment);

	//	outstring+="\n";


	if(source->child != NULL)
		writeDown(source->child);

	if(source->next  != NULL)
		writeDown(source->next);

	return 0;
}

int  exportCdcatDB::writeMedia(Node *source)
{

	medianame= QString(recode(((DBMedia *)(source->data))->name ,&out1));

	/*outstring+=QString().setNum((int)((DBMedia *)(source->data))->number)+separator;
	outstring+=QString(recode(((DBMedia *)(source->data))->owner,&out2))+separator;
	outstring+=QString(getMType(((DBMedia *)(source->data))->type))+separator;
	outstring+=QString(recode(((DBMedia *)(source->data))->modification,&out3))+separator;
	*/

	//  commentWriter(((DBMedia *)(source->data))->comment);

	if (checkAllMedia->isChecked())
	{
		if(source->child != NULL)
			writeDown(source->child);
	}
	else
	{
		// need to check if in selectedMedia
		bool inList = false;
		for (uint i=0; i < selectedMedia->count();i++)
		{
			if(selectedMedia->text(i) == medianame)
			{
				inList = true;
				break;
			}
		}
		if (inList)
		{
			if(source->child != NULL)
				writeDown(source->child);
		}

	}

	if(source->next  != NULL)
		writeDown(source->next);

	return 0;
}

int  exportCdcatDB::writeDirectory(Node *source)
{

	//   outstring+=QString(recode(((DBDirectory *)(source->data))->modification,&out3))+separator;

	//   commentWriter(((DBDirectory *)(source->data))->comment);

	if(source->child != NULL)
		writeDown(source->child);

	if(source->next  != NULL)
		writeDown(source->next);

	return 0;
}

int  exportCdcatDB::writeFile(Node *source)
{
	if(radioHtml->isChecked())
		outstring+="<tr><td><font color=\"red\">"+medianame+"</font></td><td><font color=\"blue\">";
	else
		outstring+= medianame;

	if(radioCsv->isChecked())
		outstring+=separator;

	outstring+=getFilePath(source);

	if(radioCsv->isChecked())
		outstring+=separator;

	if(radioHtml->isChecked())
		outstring+="</font></td><td align=\"right\"><font color=\"black\">";

	outstring+=QString().setNum(((DBFile *)(source->data))->size);

	outstring+=QString(getSType((((DBFile *)(source->data))->sizeType)));

	if(radioCsv->isChecked())
		outstring+=separator;

	if(radioHtml->isChecked())
		outstring+="</font></td><td align=\"right\"><font color=\"black\">";

	outstring+=QString(recode(((DBFile *)(source->data))->modification,&out3));

	if(radioCsv->isChecked())
		outstring+=separator;

	if(radioHtml->isChecked())
		outstring+="</font></td></tr>";

	outstring+="\n";

	// commentWriter(((DBFile *)(source->data))->comment);
	if(source->child != NULL)
		writeDown(source->child);

	if(((DBFile *)(source->data))->prop != NULL)
		writeDown(((DBFile *)(source->data))->prop);

	if(source->next  != NULL)
		writeDown(source->next);

	return 0;
}

int  exportCdcatDB::writeMp3Tag(Node *source)
{
	/*
	   outstring+=QString(recode(((DBMp3Tag *)(source->data))->artist,&out1));
	   outstring+=QString(recode(((DBMp3Tag *)(source->data))->title,&out2));
	   outstring+=QString(recode(((DBMp3Tag *)(source->data))->album,&out3));
	   outstring+=QString(recode(((DBMp3Tag *)(source->data))->year,&out4));
	   outstring+=QString(recode(((DBMp3Tag *)(source->data))->comment,&out5));

	   if(source->next  != NULL)
	      writeDown(source->next);
	      */
	return 0;
}

void exportCdcatDB::commentWriter(char *c)
{

	if(c == NULL)
		return;
	outstring+=QString(recode(c,&out1))+separator;

}

QString exportCdcatDB::getFilePath(Node *current)
{

	Node *up=current;
	QString a("");
	if (up != NULL)
	{
		while(up != NULL && QString(up->getNameOf())  != medianame)
		{
			a.prepend(up->getNameOf());
			a.prepend("/");
			up=up->parent;
		}
	}
	return a;

}
