/****************************************************************************
                             Hyper's CD Catalog
		A multiplatform qt and xml based catalog program

 Author    : Christoph Thielecke (crissi99@gmx.de)
 License   : GPL
 Copyright : (C) 2003 Christoph Thielecke
****************************************************************************/


#include "importdialog.h"

#include <qlabel.h>
#include <qlineedit.h>
#include <qpushbutton.h>
#include <qmessagebox.h>
#include <qtooltip.h>
#include <qwhatsthis.h>
#include <qfile.h>
#include <qfiledialog.h>
#include <stdio.h>
#include <stdlib.h>
#include <qvariant.h>


ImportDialog::ImportDialog(QWidget* parent, const char* name, bool modal, WFlags fl )
    : QDialog( parent, name, modal, fl )

{
     if ( !name )
        setName( "ImportDialog" );
    ImportDialogLayout = new QGridLayout( this, 1, 1, 11, 6, "ImportDialogLayout");

    layout4 = new QGridLayout( 0, 1, 1, 0, 6, "layout4");

    seperator_lab = new QLabel( this, "seperator_lab" );

    layout4->addWidget( seperator_lab, 2, 0 );

    seperator_lineedit = new QLineEdit( this, "seperator_lineedit" );
    seperator_lineedit->setMinimumSize( QSize( 0, 0 ) );
    seperator_lineedit->setMaximumSize( QSize( 20, 32767 ) );
    seperator_lineedit->setMaxLength ( 1 );

    layout4->addWidget( seperator_lineedit, 2, 1 );

    filename_lineedit = new QLineEdit( this, "filename_lineedit" );

    layout4->addMultiCellWidget( filename_lineedit, 1, 1, 1, 4 );

    buttonGetFile = new QPushButton( this, "buttonGetFile" );

    layout4->addWidget( buttonGetFile, 1, 5 );

    correctbadstyle = new QCheckBox( this, "correctbadstyle" );
    correctbadstyle->setText(tr("Correct bad style from gtktalog export"));

    layout4->addMultiCellWidget( correctbadstyle, 4, 4, 0, 5 );

    newdatabase = new QCheckBox( this, "newdatabase" );
    newdatabase->setText(tr("Create new Database"));

    layout4->addMultiCellWidget( newdatabase, 3, 3, 0, 5 );

    filename_lab = new QLabel( this, "filename_lab" );

    layout4->addWidget( filename_lab, 1, 0 );
    QSpacerItem* spacer = new QSpacerItem( 181, 20, QSizePolicy::Expanding, QSizePolicy::Minimum );
    layout4->addMultiCell( spacer, 5, 5, 0, 2 );

    info_lab = new QLabel( this, "info_lab" );
    info_lab->setText("info");

    layout4->addMultiCellWidget( info_lab, 0, 0, 0, 4 );

    buttonCancel = new QPushButton( this, "buttonCancel" );
    buttonCancel-> setMinimumWidth(100);

    QSpacerItem* spacer_2 = new QSpacerItem( 291, 20, QSizePolicy::Expanding, QSizePolicy::Minimum );
    layout4->addMultiCell( spacer_2, 2, 2, 2, 5 );

    buttonOK = new QPushButton( this, "buttonOK" );
    buttonOK->setDefault(true);
    buttonOK-> setMinimumWidth(100);

    layout4->addMultiCellWidget( buttonCancel, 5, 5, 4, 5 );
    layout4->addWidget( buttonOK, 5, 3 );

    ImportDialogLayout->addLayout( layout4, 0, 0 );
    languageChange();
    resize( QSize(450, 210).expandedTo(minimumSizeHint()) );
    
    setFixedSize(size());

    //clearWState( WState_Polished );

    connect(buttonOK,SIGNAL(clicked()),this,SLOT(bOk()));
    connect(buttonCancel,SIGNAL(clicked()),this,SLOT(bCan()));
    connect(buttonGetFile,SIGNAL(clicked()),this,SLOT(getFileName()));

}

/*
 *  Destroys the object and frees any allocated resources
 */
ImportDialog::~ImportDialog()
{
   //no need to delete child widgets, Qt does it for us!
}

/*
 *  Sets the strings of the subwidgets using the current
 *  language.
 */
void ImportDialog::languageChange()
{
    setCaption( tr( "Import CSV file" ) );
    filename_lab->setText( tr( "Choose a file to import:" ) );
    seperator_lineedit->setText(";");
    seperator_lab->setText( tr( "Seperator:" ) );
    QToolTip::add(seperator_lineedit , tr( "This is the seperator in dataline\n<path>SEPERATOR<size>SEPERATOR<date><space><time>" ) );
    QWhatsThis::add(seperator_lineedit , tr("Seperator in data line") );
    buttonCancel->setText( tr( "Cancel" ) );
    buttonOK->setText( tr( "Import" ) );
    buttonGetFile->setText(tr("..."));
    QToolTip::add(buttonGetFile , tr( "Open the fle dialog for selecting file to import." ) );
    QWhatsThis::add(buttonGetFile , tr("Open file dialog") );
    QToolTip::add(correctbadstyle , tr( "Corrects bad output style from gtktalog.\n<media>SEPERATOR/<dir>/SEPERATOR/<dir>SEPERATOR<\n will be to\n<media>/<dir>/<dir>" ) );
    QWhatsThis::add(correctbadstyle , tr("Corrects bad output style (gtktalog)") );
    info_lab->setText( tr("<strong>Attention! Please read README_CSV_IMPORT before you import!</strong>"));

}

int ImportDialog::bOk(void)
 {
  if((filename_lineedit->text()).isEmpty())
   {
    QMessageBox::warning(this,tr("Error:"),tr("You must be fill the \"Filename\" field!"),tr("Ok"),0);
    return 0;
   }

    if((seperator_lineedit->text()).isEmpty())
   {
    QMessageBox::warning(this,tr("Error:"),tr("You must be fill the \"Seperator\" field!"),tr("Ok"),0);
    return 0;
   }

  OK = 1;
  filename = filename_lineedit->text();
  seperator = seperator_lineedit->text();
  createdatabase = newdatabase->isChecked();

  close();
  return 0;
 }

int ImportDialog::bCan(void)
 {
  OK = 0;
  close();
  return 0;
 }

 void ImportDialog::getFileName()
 {
    filename_lineedit->setText(QFileDialog::getOpenFileName(
				 getenv("HOME"),
						  //     "/data/compile/CdCat-0.91/",
						       tr ("csv files(*.csv)"),
						       0,
						       tr ("open file dialog"),
						       tr ("Choose a file for import")));

 }
